@@ -149,18 +149,17 @@ def getSitesWithData(siteMapper, ddmIF, datasetName, prodsourcelabel, job_label,
 
 
 # get nuclei where data is available
-def getNucleiWithData(siteMapper,ddmIF,datasetName,candidateNuclei=[],deepScan=False):
+def getNucleiWithData(siteMapper, ddmIF, datasetName, candidateNuclei, deepScan=False):
     # get replicas
     try:
-        replicaMap = ddmIF.listReplicasPerDataset(datasetName,deepScan)
+        replicaMap = ddmIF.listReplicasPerDataset(datasetName, deepScan)
     except Exception:
         errtype,errvalue = sys.exc_info()[:2]
         return errtype,'ddmIF.listReplicasPerDataset failed with %s' % errvalue
     # loop over all clouds
     retMap = {}
-    for tmpNucleus,tmpNucleusSpec in iteritems(siteMapper.nuclei):
-        if candidateNuclei != [] and tmpNucleus not in candidateNuclei:
-            continue
+    for tmpNucleus in candidateNuclei:
+        tmpNucleusSpec = siteMapper.getNucleus(tmpNucleus)
         # loop over all datasets
         totalNum = 0
         totalSize = 0
@@ -228,7 +227,7 @@ def getAnalSitesWithData(siteList,siteMapper,ddmIF,datasetName):
     # get replicas
     try:
         replicaMap= {}
-        replicaMap[datasetName] = ddmIF.listDatasetReplicas(datasetName, use_vp=True)
+        replicaMap[datasetName] = ddmIF.listDatasetReplicas(datasetName, use_vp=True, skip_incomplete_element=True)
     except Exception:
         errtype,errvalue = sys.exc_info()[:2]
         return errtype,'ddmIF.listDatasetReplicas failed with %s' % errvalue
@@ -488,8 +487,11 @@ def getDictToSetNucleus(nucleusSpec,tmpDatasetSpecs):
         # skip distributed datasets
         if DataServiceUtils.getDistributedDestination(datasetSpec.storageToken) is not None:
             continue
-        # get token
+        # get endpoint relevant to token
         endPoint = nucleusSpec.getAssociatedEndpoint(datasetSpec.storageToken)
+        if endPoint is None and not nucleusSpec.is_nucleus() and nucleusSpec.get_default_endpoint_out():
+            # use default endpoint for satellite that doesn't have relevant endpoint
+            endPoint = nucleusSpec.get_default_endpoint_out()
         if endPoint is None:
             continue
         token = endPoint['ddm_endpoint_name']
@@ -761,17 +763,204 @@ def getUsersJobsStats(tbIF, vo, prod_source_label, cache_lifetime=60):
     return ret_val, ret_map
 
 
+# get gshare usage
+def getGShareUsage(tbIF, gshare, fresher_than_minutes_ago=15):
+    # initialize
+    ret_val = False
+    ret_map = {}
+    # timestamps
+    current_time = datetime.datetime.utcnow()
+    # try some times
+    for _ in range(99):
+        now_time = datetime.datetime.utcnow()
+        # skip if too long after original current time
+        if now_time - current_time > datetime.timedelta(seconds=max(3, fresher_than_minutes_ago/3)):
+            # break trying
+            break
+        try:
+            # query from PanDA DB directly
+            sql_get_gshare = (
+                    """SELECT m.value_json """
+                    """FROM ATLAS_PANDA.Metrics m """
+                    """WHERE m.metric=:metric """
+                        """AND m.gshare=:gshare """
+                        """AND m.timestamp>=:min_timestamp """
+                )
+            # varMap
+            varMap = {
+                    ':metric': 'gshare_preference',
+                    ':gshare': gshare,
+                    ':min_timestamp': now_time - datetime.timedelta(minutes=fresher_than_minutes_ago),
+                }
+            # result
+            res = tbIF.querySQL(sql_get_gshare, varMap)
+            if res:
+                value_json = res[0][0]
+                # json of data
+                ret_map = json.loads(value_json)
+                # make True return
+                ret_val = True
+            # break trying
+            break
+        except Exception as e:
+            # dump error message
+            err_str = 'AtlasBrokerUtils.getGShareUsage got {0}: {1} \n'.format(e.__class__.__name__, e)
+            sys.stderr.write(err_str)
+            # break trying
+            break
+    # return
+    return ret_val, ret_map
+
+
+# get user evaluation
+def getUserEval(tbIF, user, fresher_than_minutes_ago=20):
+    # initialize
+    ret_val = False
+    ret_map = {}
+    # timestamps
+    current_time = datetime.datetime.utcnow()
+    # try some times
+    for _ in range(99):
+        now_time = datetime.datetime.utcnow()
+        # skip if too long after original current time
+        if now_time - current_time > datetime.timedelta(seconds=max(3, fresher_than_minutes_ago/3)):
+            # break trying
+            break
+        try:
+            # query from PanDA DB directly
+            sql_get_user_eval = (
+                    """SELECT m.value_json."{user}" """
+                    """FROM ATLAS_PANDA.Metrics m """
+                    """WHERE m.metric=:metric """
+                        """AND m.timestamp>=:min_timestamp """
+                ).format(user=user)
+            # varMap
+            varMap = {
+                    ':metric': 'analy_user_eval',
+                    ':min_timestamp': now_time - datetime.timedelta(minutes=fresher_than_minutes_ago),
+                }
+            # result
+            res = tbIF.querySQL(sql_get_user_eval, varMap)
+            if res:
+                value_json = res[0][0]
+                # json of data
+                ret_map = json.loads(value_json) if value_json else None
+                # make True return
+                ret_val = True
+            # break trying
+            break
+        except Exception as e:
+            # dump error message
+            err_str = 'AtlasBrokerUtils.getUserEval got {0}: {1} \n'.format(e.__class__.__name__, e)
+            sys.stderr.write(err_str)
+            # break trying
+            break
+    # return
+    return ret_val, ret_map
+
+
+# get user task evaluation
+def getUserTaskEval(tbIF, taskID, fresher_than_minutes_ago=15):
+    # initialize
+    ret_val = False
+    ret_map = {}
+    # timestamps
+    current_time = datetime.datetime.utcnow()
+    # try some times
+    for _ in range(99):
+        now_time = datetime.datetime.utcnow()
+        # skip if too long after original current time
+        if now_time - current_time > datetime.timedelta(seconds=max(3, fresher_than_minutes_ago/3)):
+            # break trying
+            break
+        try:
+            # query from PanDA DB directly
+            sql_get_task_eval = (
+                    """SELECT tev.value_json """
+                    """FROM ATLAS_PANDA.Task_Evaluation tev """
+                    """WHERE tev.metric=:metric """
+                        """AND tev.jediTaskID=:taskID """
+                        """AND tev.timestamp>=:min_timestamp """
+                )
+            # varMap
+            varMap = {
+                    ':metric': 'analy_task_eval',
+                    ':taskID': taskID,
+                    ':min_timestamp': now_time - datetime.timedelta(minutes=fresher_than_minutes_ago),
+                }
+            # result
+            res = tbIF.querySQL(sql_get_task_eval, varMap)
+            if res:
+                value_json = res[0][0]
+                # json of data
+                ret_map = json.loads(value_json) if value_json else None
+                # make True return
+                ret_val = True
+            # break trying
+            break
+        except Exception as e:
+            # dump error message
+            err_str = 'AtlasBrokerUtils.getUserTaskEval got {0}: {1} \n'.format(e.__class__.__name__, e)
+            sys.stderr.write(err_str)
+            # break trying
+            break
+    # return
+    return ret_val, ret_map
+
+
+# get analysis sites class
+def getAnalySitesClass(tbIF, fresher_than_minutes_ago=60):
+    # initialize
+    ret_val = False
+    ret_map = {}
+    # timestamps
+    current_time = datetime.datetime.utcnow()
+    # try some times
+    for _ in range(99):
+        now_time = datetime.datetime.utcnow()
+        # skip if too long after original current time
+        if now_time - current_time > datetime.timedelta(seconds=max(3, fresher_than_minutes_ago/3)):
+            # break trying
+            break
+        try:
+            # query from PanDA DB directly
+            sql_get_task_eval = (
+                    """SELECT m.computingSite, m.value_json.class """
+                    """FROM ATLAS_PANDA.Metrics m """
+                    """WHERE m.metric=:metric """
+                        """AND m.timestamp>=:min_timestamp """
+                )
+            # varMap
+            varMap = {
+                    ':metric': 'analy_site_eval',
+                    ':min_timestamp': now_time - datetime.timedelta(minutes=fresher_than_minutes_ago),
+                }
+            # result
+            res = tbIF.querySQL(sql_get_task_eval, varMap)
+            if res:
+                for site, class_value in res:
+                    ret_map[site] = int(class_value)
+                    # make True return
+                ret_val = True
+            # break trying
+            break
+        except Exception as e:
+            # dump error message
+            err_str = 'AtlasBrokerUtils.getAnalySitesEval got {0}: {1} \n'.format(e.__class__.__name__, e)
+            sys.stderr.write(err_str)
+            # break trying
+            break
+    # return
+    return ret_val, ret_map
+
+
 # check SW with json
 class JsonSoftwareCheck:
     # constructor
-    def __init__(self, site_mapper, json_name=None):
-        if json_name is None:
-            json_name = '/cvmfs/atlas.cern.ch/repo/sw/local/etc/cric_pandaqueue_tags.json'
+    def __init__(self, site_mapper, sw_map):
+
         self.siteMapper = site_mapper
-        try:
-            self.swDict = json.load(open(json_name))
-        except Exception:
-            self.swDict = dict()
+        self.sw_map = sw_map
 
     # get lists
     def check(self, site_list, cvmfs_tag, sw_project, sw_version, cmt_config, need_cvmfs, cmt_config_only,
@@ -781,13 +970,13 @@ class JsonSoftwareCheck:
         noAutoSite = []
         for tmpSiteName in site_list:
             tmpSiteSpec = self.siteMapper.getSite(tmpSiteName)
-            if tmpSiteSpec.releases == ['AUTO'] and tmpSiteName in self.swDict:
+            if tmpSiteSpec.releases == ['AUTO'] and tmpSiteName in self.sw_map:
                 try:
                     go_ahead = False
                     # convert to a dict
                     architecture_map = {}
-                    if 'architectures' in self.swDict[tmpSiteName]:
-                        for arch_spec in self.swDict[tmpSiteName]['architectures']:
+                    if 'architectures' in self.sw_map[tmpSiteName]:
+                        for arch_spec in self.sw_map[tmpSiteName]['architectures']:
                             if 'type' in arch_spec:
                                 architecture_map[arch_spec['type']] = arch_spec
                     # check if need CPU
@@ -883,27 +1072,27 @@ class JsonSoftwareCheck:
                 # check for fat container
                 if container_name:
                     # check for container
-                    if not only_tags_fc and ('any' in self.swDict[tmpSiteName]["containers"] or
-                            '/cvmfs' in self.swDict[tmpSiteName]["containers"]):
+                    if not only_tags_fc and ('any' in self.sw_map[tmpSiteName]["containers"] or
+                            '/cvmfs' in self.sw_map[tmpSiteName]["containers"]):
                         # any in containers
                         okSite.append(tmpSiteName)
-                    elif container_name in set([t['container_name'] for t in self.swDict[tmpSiteName]['tags']
+                    elif container_name in set([t['container_name'] for t in self.sw_map[tmpSiteName]['tags']
                                                 if t['container_name']]):
                         # logical name in tags or any in containers
                         okSite.append(tmpSiteName)
-                    elif container_name in set([s for t in self.swDict[tmpSiteName]['tags'] for s in t['sources']
+                    elif container_name in set([s for t in self.sw_map[tmpSiteName]['tags'] for s in t['sources']
                                                if t['sources']]):
                         # full path in sources
                         okSite.append(tmpSiteName)
                     elif not only_tags_fc:
                         # get sources in all tag list
-                        if 'ALL' in self.swDict:
-                            source_list_in_all_tag = [s for t in self.swDict['ALL']['tags']
+                        if 'ALL' in self.sw_map:
+                            source_list_in_all_tag = [s for t in self.sw_map['ALL']['tags']
                                                       for s in t['sources'] if t['container_name']==container_name]
                         else:
                             source_list_in_all_tag = []
                         # prefix with full path
-                        for tmp_prefix in self.swDict[tmpSiteName]["containers"]:
+                        for tmp_prefix in self.sw_map[tmpSiteName]["containers"]:
                             if container_name.startswith(tmp_prefix):
                                 okSite.append(tmpSiteName)
                                 break
@@ -918,22 +1107,22 @@ class JsonSoftwareCheck:
                     continue
                 # only cmt config check
                 if cmt_config_only:
-                    if not cmt_config or cmt_config in self.swDict[tmpSiteName]['cmtconfigs']:
+                    if not cmt_config or cmt_config in self.sw_map[tmpSiteName]['cmtconfigs']:
                         okSite.append(tmpSiteName)
                     continue
                 # check if CVMFS is available
-                if 'any' in self.swDict[tmpSiteName]["cvmfs"] or cvmfs_tag in self.swDict[tmpSiteName]["cvmfs"]:
+                if 'any' in self.sw_map[tmpSiteName]["cvmfs"] or cvmfs_tag in self.sw_map[tmpSiteName]["cvmfs"]:
                     # check if container is available
-                    if 'any' in self.swDict[tmpSiteName]["containers"] or \
-                            '/cvmfs' in self.swDict[tmpSiteName]["containers"]:
+                    if 'any' in self.sw_map[tmpSiteName]["containers"] or \
+                            '/cvmfs' in self.sw_map[tmpSiteName]["containers"]:
                         okSite.append(tmpSiteName)
                     # check cmt config
-                    elif not need_container and cmt_config in self.swDict[tmpSiteName]['cmtconfigs']:
+                    elif not need_container and cmt_config in self.sw_map[tmpSiteName]['cmtconfigs']:
                         okSite.append(tmpSiteName)
                 elif not need_cvmfs:
-                    if not need_container or 'any' in self.swDict[tmpSiteName]["containers"]:
+                    if not need_container or 'any' in self.sw_map[tmpSiteName]["containers"]:
                         # check tags
-                        for tag in self.swDict[tmpSiteName]["tags"]:
+                        for tag in self.sw_map[tmpSiteName]["tags"]:
                             if tag['cmtconfig'] == cmt_config and tag['project'] == sw_project \
                                and tag['release'] == sw_version:
                                 okSite.append(tmpSiteName)
