@@ -1,5 +1,7 @@
 import sys
 import random
+import copy
+import math
 import traceback
 
 from six import iteritems
@@ -255,13 +257,46 @@ class AtlasProdTaskBrokerThread (WorkerThread):
         self.prioRW       = prioRW
         self.numTasks     = 0
         self.workQueue    = workQueue
+        self.summaryList = None
 
+    # init summary list
+    def init_summary_list(self, header, comment, initial_list):
+        self.summaryList = []
+        self.summaryList.append('===== {} ====='.format(header))
+        if comment:
+            self.summaryList.append(comment)
+        self.summaryList.append('the number of initial candidates: {}'.format(len(initial_list)))
 
-    # wrapper for return
-    def sendLogMessage(self,tmpLog):
-        # send info to logger
-        #tmpLog.bulkSendMsg('taskbrokerage',loggerName='bamboo')
-        tmpLog.debug('sent')
+    # dump summary
+    def dump_summary(self, tmp_log, final_candidates=None):
+        if not self.summaryList:
+            return
+        tmp_log.info('')
+        for m in self.summaryList:
+            tmp_log.info(m)
+        if not final_candidates:
+            final_candidates = []
+        tmp_log.info('the number of final candidates: {}'.format(len(final_candidates)))
+        tmp_log.info('')
+
+    # make summary
+    def add_summary_message(self, old_list, new_list, message):
+        if old_list and len(old_list) != len(new_list):
+            red = int(math.ceil(((len(old_list) - len(new_list)) * 100) / len(old_list)))
+            self.summaryList.append('{:>5} -> {:>3} candidates, {:>3}% cut : {}'.format(len(old_list),
+                                                                                        len(new_list),
+                                                                                        red, message))
+
+    # post-process for errors
+    def post_process_for_error(self, task_spec, tmp_log, msg, dump_summary=True):
+        if dump_summary:
+            self.dump_summary(tmp_log)
+        tmp_log.error(msg)
+        task_spec.resetChangedList()
+        task_spec.setErrDiag(tmp_log.uploadLog(task_spec.jediTaskID))
+        self.taskBufferIF.updateTask_JEDI(task_spec, {'jediTaskID': task_spec.jediTaskID},
+                                          oldStatus=['assigning'], updateDEFT=False,
+                                          setFrozenTime=False)
 
 
     # main function
@@ -320,23 +355,36 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                     # make logger
                     tmpLog = MsgWrapper(self.logger,'<jediTaskID={0}>'.format(taskSpec.jediTaskID),monToken='jediTaskID={0}'.format(taskSpec.jediTaskID))
                     tmpLog.debug('start')
-                    tmpLog.info('thrInputSize:{0} thrInputNum:{1} thrInputSizeFrac:{2} thrInputNumFrac;{3}'.format(thrInputSize,
-                                                                                                                    thrInputNum,
-                                                                                                                    thrInputSizeFrac,
-                                                                                                                    thrInputNumFrac))
+                    tmpLog.info('thrInputSize:{} thrInputNum:{} thrInputSizeFrac:{} thrInputNumFrac:{}'.format(thrInputSize,
+                                                                                                               thrInputNum,
+                                                                                                               thrInputSizeFrac,
+                                                                                                               thrInputNumFrac))
+                    tmpLog.info('full-chain:{}'.format(taskSpec.get_full_chain()))
                     # read task parameters
                     try:
                         taskParam = self.taskBufferIF.getTaskParamsWithID_JEDI(taskSpec.jediTaskID)
                         taskParamMap = RefinerUtils.decodeJSON(taskParam)
                     except Exception:
                         tmpLog.error('failed to read task params')
+                        taskSpec.resetChangedList()
                         taskSpec.setErrDiag(tmpLog.uploadLog(taskSpec.jediTaskID))
-                        self.sendLogMessage(tmpLog)
+                        self.taskBufferIF.updateTask_JEDI(taskSpec, {'jediTaskID': taskSpec.jediTaskID},
+                                                          oldStatus=['assigning'], updateDEFT=False,
+                                                          setFrozenTime=False)
                         continue
                     # RW
                     taskRW = self.taskBufferIF.calculateTaskWorldRW_JEDI(taskSpec.jediTaskID)
                     # get nuclei
-                    nucleusList = siteMapper.nuclei
+                    nucleusList = copy.copy(siteMapper.nuclei)
+                    if taskSpec.get_full_chain():
+                        # use satellites with bareNucleus as nuclei for full chain
+                        for tmpNucleus, tmpNucleusSpec in iteritems(siteMapper.satellites):
+                            if tmpNucleusSpec.get_bare_nucleus_mode():
+                                nucleusList[tmpNucleus] = tmpNucleusSpec
+                    # init summary list
+                    self.init_summary_list('Task brokerage summary',
+                                           None,
+                                           nucleusList)
                     if taskSpec.nucleus in siteMapper.nuclei:
                         candidateNucleus = taskSpec.nucleus
                     elif taskSpec.nucleus in siteMapper.satellites:
@@ -347,6 +395,7 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                         ######################################
                         # check status
                         newNucleusList = {}
+                        oldNucleusList = copy.copy(nucleusList)
                         for tmpNucleus,tmpNucleusSpec in iteritems(nucleusList):
                             if tmpNucleusSpec.state not in ['ACTIVE']:
                                 tmpLog.info('  skip nucleus={0} due to status={1} criteria=-status'.format(tmpNucleus,
@@ -355,10 +404,9 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                                 newNucleusList[tmpNucleus] = tmpNucleusSpec
                         nucleusList = newNucleusList
                         tmpLog.info('{0} candidates passed status check'.format(len(nucleusList)))
-                        if nucleusList == {}:
-                            tmpLog.error('no candidates')
-                            taskSpec.setErrDiag(tmpLog.uploadLog(taskSpec.jediTaskID))
-                            self.sendLogMessage(tmpLog)
+                        self.add_summary_message(oldNucleusList, nucleusList, 'status check')
+                        if not nucleusList:
+                            self.post_process_for_error(taskSpec, tmpLog, 'no candidates')
                             continue
                         ######################################
                         # check status of transfer backlog
@@ -367,6 +415,7 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                             tmpLog.info('skip transfer backlog check due to negative T1Weight')
                         else:
                             newNucleusList = {}
+                            oldNucleusList = copy.copy(nucleusList)
                             backlogged_nuclei = self.taskBufferIF.getBackloggedNuclei()
                             for tmpNucleus, tmpNucleusSpec in iteritems(nucleusList):
                                 if tmpNucleus in backlogged_nuclei:
@@ -376,20 +425,25 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                                     newNucleusList[tmpNucleus] = tmpNucleusSpec
                             nucleusList = newNucleusList
                             tmpLog.info('{0} candidates passed transfer backlog check'.format(len(nucleusList)))
-                            if nucleusList == {}:
-                                tmpLog.error('no candidates')
-                                taskSpec.setErrDiag(tmpLog.uploadLog(taskSpec.jediTaskID))
-                                self.sendLogMessage(tmpLog)
+                            self.add_summary_message(oldNucleusList, nucleusList, 'transfer backlog check')
+                            if not nucleusList:
+                                self.post_process_for_error(taskSpec, tmpLog, 'no candidates')
                                 continue
                         ######################################
                         # check endpoint
                         fractionFreeSpace = {}
                         newNucleusList = {}
+                        oldNucleusList = copy.copy(nucleusList)
                         tmpStat,tmpDatasetSpecList = self.taskBufferIF.getDatasetsWithJediTaskID_JEDI(taskSpec.jediTaskID,
                                                                                                       ['output','log'])
-                        for tmpNucleus,tmpNucleusSpec in iteritems(nucleusList):
+                        for tmpNucleus, tmpNucleusSpec in iteritems(nucleusList):
                             toSkip = False
+                            origNucleusSpec = tmpNucleusSpec
                             for tmpDatasetSpec in tmpDatasetSpecList:
+                                tmpNucleusSpec = origNucleusSpec
+                                # use secondary nucleus for full-chain if defined
+                                if taskSpec.get_full_chain() and tmpNucleusSpec.get_secondary_nucleus():
+                                    tmpNucleusSpec = siteMapper.getNucleus(tmpNucleusSpec.get_secondary_nucleus())
                                 # ignore distributed datasets
                                 if DataServiceUtils.getDistributedDestination(tmpDatasetSpec.storageToken) is not None:
                                     continue
@@ -410,7 +464,11 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                                     break
                                 """
                                 # check space
-                                tmpSpaceSize = tmpEP['space_free'] + tmpEP['space_expired']
+                                tmpSpaceSize = 0
+                                if tmpEP['space_free']:
+                                    tmpSpaceSize += tmpEP['space_free']
+                                if tmpEP['space_expired']:
+                                    tmpSpaceSize += tmpEP['space_expired']
                                 tmpSpaceToUse = 0
                                 if tmpNucleus in self.fullRW:
                                     # 0.25GB per cpuTime/corePower/day
@@ -439,17 +497,17 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                                     fractionFreeSpace[tmpNucleus] = {'total':tmpEP['space_total'],
                                                                      'free':tmpSpaceSize-tmpSpaceToUse}
                             if not toSkip:
-                                newNucleusList[tmpNucleus] = tmpNucleusSpec
+                                newNucleusList[tmpNucleus] = origNucleusSpec
                         nucleusList = newNucleusList
                         tmpLog.info('{0} candidates passed endpoint check {1} TB'.format(len(nucleusList),diskThreshold/1024))
-                        if nucleusList == {}:
-                            tmpLog.error('no candidates')
-                            taskSpec.setErrDiag(tmpLog.uploadLog(taskSpec.jediTaskID))
-                            self.sendLogMessage(tmpLog)
+                        self.add_summary_message(oldNucleusList, nucleusList, 'storage endpoint check')
+                        if not nucleusList:
+                            self.post_process_for_error(taskSpec, tmpLog, 'no candidates')
                             continue
                         ######################################
                         # ability to execute jobs
                         newNucleusList = {}
+                        oldNucleusList = copy.copy(nucleusList)
                         # get all panda sites
                         tmpSiteList = []
                         for tmpNucleus,tmpNucleusSpec in iteritems(nucleusList):
@@ -462,8 +520,11 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                         tmpLog.debug('===== done for job check')
                         if tmpSt != Interaction.SC_SUCCEEDED:
                             tmpLog.error('no sites can run jobs')
+                            taskSpec.resetChangedList()
                             taskSpec.setErrDiag(tmpLog.uploadLog(taskSpec.jediTaskID))
-                            self.sendLogMessage(tmpLog)
+                            self.taskBufferIF.updateTask_JEDI(taskSpec, {'jediTaskID': taskSpec.jediTaskID},
+                                                              oldStatus=['assigning'], updateDEFT=False,
+                                                              setFrozenTime=False)
                             continue
                         okNuclei = set()
                         for tmpSite in tmpRet:
@@ -476,10 +537,9 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                                 tmpLog.info('  skip nucleus={0} due to missing ability to run jobs criteria=-job'.format(tmpNucleus))
                         nucleusList = newNucleusList
                         tmpLog.info('{0} candidates passed job check'.format(len(nucleusList)))
-                        if nucleusList == {}:
-                            tmpLog.error('no candidates')
-                            taskSpec.setErrDiag(tmpLog.uploadLog(taskSpec.jediTaskID))
-                            self.sendLogMessage(tmpLog)
+                        self.add_summary_message(oldNucleusList, nucleusList, 'job check')
+                        if not nucleusList:
+                            self.post_process_for_error(taskSpec, tmpLog, 'no candidates')
                             continue
                         ######################################
                         # data locality
@@ -509,9 +569,10 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                                                                               list(nucleusList.keys()),
                                                                               deepScan)
                             if tmpSt != Interaction.SC_SUCCEEDED:
-                                tmpLog.error('failed to get nuclei where data is available, since {0}'.format(tmpRet))
-                                taskSpec.setErrDiag(tmpLog.uploadLog(taskSpec.jediTaskID))
-                                self.sendLogMessage(tmpLog)
+                                self.post_process_for_error(taskSpec, tmpLog,
+                                                            'failed to get nuclei where data is available, '
+                                                            'since {0}'.format(tmpRet),
+                                                            False)
                                 toSkip = True
                                 break
                             # sum
@@ -524,6 +585,7 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                             continue
                         if availableData != {}:
                             newNucleusList = {}
+                            oldNucleusList = copy.copy(nucleusList)
                             # skip if no data
                             skipMsgList = []
                             for tmpNucleus,tmpNucleusSpec in iteritems(nucleusList):
@@ -575,12 +637,54 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                                     '  the following conditions required to disable data locality check: {}'.format(
                                         data_locality_check_str))
                             tmpLog.info('{0} candidates passed data check'.format(len(nucleusList)))
-                            if nucleusList == {}:
-                                tmpLog.error('no candidates')
-                                taskSpec.setErrDiag(tmpLog.uploadLog(taskSpec.jediTaskID))
-                                self.sendLogMessage(tmpLog)
+                            self.add_summary_message(oldNucleusList, nucleusList, 'data check')
+                            if not nucleusList:
+                                self.post_process_for_error(taskSpec, tmpLog, 'no candidates')
                                 continue
                         ######################################
+                        # check for full chain
+                        newNucleusList = {}
+                        oldNucleusList = copy.copy(nucleusList)
+                        parent_full_chain = False
+                        if taskSpec.get_full_chain() and taskSpec.jediTaskID != taskSpec.parent_tid:
+                            tmpStat, parentTaskSpec = self.taskBufferIF.getTaskWithID_JEDI(taskSpec.parent_tid, False)
+
+                            if not tmpStat or parentTaskSpec is None:
+                                self.post_process_for_error(taskSpec, tmpLog,
+                                                            'failed to get parent taskSpec with '
+                                                            'jediTaskID={0}'.format(taskSpec.parent_tid),
+                                                            False)
+                                continue
+                            parent_full_chain = parentTaskSpec.check_full_chain_with_nucleus(
+                                siteMapper.getNucleus(parentTaskSpec.nucleus))
+                        for tmpNucleus, tmpNucleusSpec in iteritems(nucleusList):
+                            # nucleus to run only full-chain tasks
+                            if tmpNucleusSpec.get_bare_nucleus_mode() == 'only' and taskSpec.get_full_chain() is None:
+                                tmpLog.info('  skip nucleus={} since only full-chain tasks are '
+                                            'allowed criteria=-full_chain'.format(tmpNucleus))
+                                continue
+                            # task requirement to run on nucleus with full-chain support
+                            if tmpNucleusSpec.get_bare_nucleus_mode() is None and \
+                                (taskSpec.check_full_chain_with_mode('only') or
+                                 taskSpec.check_full_chain_with_mode('require')):
+                                tmpLog.info('  skip nucleus={} due to lack of full-chain capability '
+                                            'criteria=-full_chain'.format(tmpNucleus))
+                                continue
+                            # check parent task
+                            if taskSpec.get_full_chain() and parent_full_chain:
+                                if tmpNucleus != parentTaskSpec.nucleus:
+                                    tmpLog.info('  skip nucleus={} since the parent of the full-chain ran elsewhere '
+                                                'criteria=-full_chain'.format(tmpNucleus))
+                                    continue
+                            newNucleusList[tmpNucleus] = tmpNucleusSpec
+                        nucleusList = newNucleusList
+                        tmpLog.info('{0} candidates passed full-chain check'.format(len(nucleusList)))
+                        self.add_summary_message(oldNucleusList, nucleusList, 'full-chain check')
+                        if not nucleusList:
+                            self.post_process_for_error(taskSpec, tmpLog, 'no candidates')
+                            continue
+                        self.dump_summary(tmpLog, nucleusList)
+                        ######################################
                         # weight
                         self.prioRW.acquire()
                         nucleusRW = self.prioRW[taskSpec.currentPriority]
@@ -592,7 +696,7 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                                 nucleusRW[tmpNucleus] = 0
                             wStr = '1'
                             # with RW
-                            if tmpNucleus in nucleusRW and nucleusRW[tmpNucleus] >= cutOffRW:
+                            if tmpNucleus in nucleusRW and nucleusRW[tmpNucleus] and nucleusRW[tmpNucleus] >= cutOffRW:
                                 weight = 1 / float(nucleusRW[tmpNucleus])
                                 wStr += '/( RW={0} )'.format(nucleusRW[tmpNucleus])
                             else:
@@ -622,7 +726,6 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                             tmpLog.info('  use nucleus={0} weight={1} {2} criteria=+use'.format(tmpNucleus,weight,wStr))
                             totalWeight += weight
                             nucleusweights.append((tmpNucleus,weight))
-                        tmpLog.info('final {0} candidates'.format(len(nucleusList)))
                         ######################################
                         # final selection
                         tgtWeight = random.uniform(0,totalWeight)
@@ -644,7 +747,6 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                     retMap = {taskSpec.jediTaskID: AtlasBrokerUtils.getDictToSetNucleus(nucleusSpec,tmpDatasetSpecs)}
                     tmpRet = self.taskBufferIF.setCloudToTasks_JEDI(retMap)
                     tmpLog.info('  set nucleus={0} with {1} criteria=+set'.format(candidateNucleus,tmpRet))
-                    self.sendLogMessage(tmpLog)
                     if tmpRet:
                         tmpMsg = 'set task_status=ready'
                         tmpLog.sendMsg(tmpMsg,self.msgType)
@@ -654,6 +756,8 @@ class AtlasProdTaskBrokerThread (WorkerThread):
                         if prio > taskSpec.currentPriority:
                             continue
                         if candidateNucleus in rwMap:
+                            if not rwMap[candidateNucleus]:
+                                rwMap[candidateNucleus] = 0
                             rwMap[candidateNucleus] += taskRW
                         else:
                             rwMap[candidateNucleus] = taskRW
