@@ -63,18 +63,19 @@ class JediTaskBuffer(TaskBuffer.TaskBuffer, CommandReceiveInterface):
                                    nFilesPerJob,nEventsPerRange,nChunksForScout,includePatt,
                                    excludePatt,xmlConfig,noWaitParent,parent_tid,pid,maxFailure,
                                    useRealNumEvents,respectLB,tgtNumEventsPerJob,skipFilesUsedBy,
-                                   ramCount,taskSpec,skipShortInput, inputPreStaging):
+                                   ramCount,taskSpec,skipShortInput, inputPreStaging, order_by):
         with self.proxyPool.get() as proxy:
             return proxy.insertFilesForDataset_JEDI(datasetSpec,fileMap,datasetState,stateUpdateTime,
-                                                  nEventsPerFile,nEventsPerJob,maxAttempt,
-                                                  firstEventNumber,nMaxFiles,nMaxEvents,
-                                                  useScout,fileList,useFilesWithNewAttemptNr,
-                                                  nFilesPerJob,nEventsPerRange,nChunksForScout,
-                                                  includePatt,excludePatt,xmlConfig,
-                                                  noWaitParent,parent_tid,pid,maxFailure,
-                                                  useRealNumEvents,respectLB,
-                                                  tgtNumEventsPerJob,skipFilesUsedBy,
-                                                  ramCount,taskSpec,skipShortInput, inputPreStaging)
+                                                    nEventsPerFile,nEventsPerJob,maxAttempt,
+                                                    firstEventNumber,nMaxFiles,nMaxEvents,
+                                                    useScout,fileList,useFilesWithNewAttemptNr,
+                                                    nFilesPerJob,nEventsPerRange,nChunksForScout,
+                                                    includePatt,excludePatt,xmlConfig,
+                                                    noWaitParent,parent_tid,pid,maxFailure,
+                                                    useRealNumEvents,respectLB,
+                                                    tgtNumEventsPerJob,skipFilesUsedBy,
+                                                    ramCount,taskSpec,skipShortInput, inputPreStaging,
+                                                    order_by)
 
     # get files from the JEDI contents table with jediTaskID and/or datasetID
     def getFilesInDatasetWithID_JEDI(self,jediTaskID=None,datasetID=None,nFiles=None,status=None):
@@ -679,16 +680,6 @@ class JediTaskBuffer(TaskBuffer.TaskBuffer, CommandReceiveInterface):
         with self.proxyPool.get() as proxy:
             return proxy.getTaskWithJumbo_JEDI(vo, prodSourceLabel)
 
-    # get tasks to enabme jumbo jobs
-    def getTaskToEnableJumbo_JEDI(self, vo, prodSourceLabel, maxPrio, nEventsToEnable):
-        with self.proxyPool.get() as proxy:
-            return proxy.getTaskToEnableJumbo_JEDI(vo, prodSourceLabel, maxPrio, nEventsToEnable)
-
-    # get release and cache at jumbo job enabled sites
-    def getRelCacheForJumbo_JEDI(self):
-        with self.proxyPool.get() as proxy:
-            return proxy.getRelCacheForJumbo_JEDI()
-
     # kick pending tasks with jumbo jobs
     def kickPendingTasksWithJumbo_JEDI(self, jediTaskID):
         with self.proxyPool.get() as proxy:
@@ -803,3 +794,10 @@ class JediTaskBuffer(TaskBuffer.TaskBuffer, CommandReceiveInterface):
     def setMissingFilesAboutIdds_JEDI(self, jeditaskid, filenames):
         with self.proxyPool.get() as proxy:
             return proxy.setMissingFilesAboutIdds_JEDI(jeditaskid, filenames)
+
+    # set missing files according to iDDS messages
+    def load_sw_map(self):
+        with self.proxyPool.get() as proxy:
+            return proxy.load_sw_map()
+
+
