@@ -1,3 +1,4 @@
+import math
 import re
 import enum
 
@@ -56,11 +57,13 @@ class JediTaskSpec(object):
         'disableAutoRetry'   : 'DR',
         'dynamicNumEvents'   : 'DY',
         'nEsConsumers'       : 'EC',
+        'nEventsPerInput'    : 'EI',
         'encJobParams'       : 'EJ',
         'nEventsPerWorker'   : 'ES',
         'firstContentsFeed'  : 'FC',
         'failGoalUnreached'  : 'FG',
         'firstEvent'         : 'FT',
+        'fullChain'          : 'FU',
         'groupBoundaryID'    : 'GB',
         'hpoWorkflow'        : 'HO',
         'instantiateTmplSite': 'IA',
@@ -68,6 +71,7 @@ class JediTaskSpec(object):
         'ipStack'            : 'IK',
         'allowInputLAN'      : 'IL',
         'ignoreMissingInDS'  : 'IM',
+        'intermediateTask'   : 'IN',
         'ipConnectivity'     : 'IP',
         'inputPreStaging'    : 'IS',
         'instantiateTmpl'    : 'IT',
@@ -84,6 +88,7 @@ class JediTaskSpec(object):
         'mergeOutput'        : 'MO',
         'multiStepExec'      : 'MS',
         'maxWalltime'        : 'MW',
+        'maxEventsPerJob'    : 'MX',
         'noExecStrCnv'       : 'NC',
         'notDiscardEvents'   : 'ND',
         'nEventsPerJob'      : 'NE',
@@ -94,6 +99,7 @@ class JediTaskSpec(object):
         'nSitesPerJob'       : 'NS',
         'nChunksToWait'      : 'NT',
         'noWaitParent'       : 'NW',
+        'orderInputBy'       : 'OI',
         'orderByLB'          : 'OL',
         'onSiteMerging'      : 'OM',
         'osMatching'         : 'OS',
@@ -110,6 +116,8 @@ class JediTaskSpec(object):
         'releasePerLB'       : 'RP',
         'respectSplitRule'   : 'RR',
         'randomSeed'         : 'RS',
+        'retryRamOffset'     : 'RX',
+        'retryRamStep'       : 'RY',
         'resurrectConsumers' : 'SC',
         'switchEStoNormal'   : 'SE',
         'stayOutputOnSite'   : 'SO',
@@ -186,6 +194,15 @@ class JediTaskSpec(object):
     enum_inputPreStaging = {'use': '1',
                             'notUse': '0'}
 
+    # enum for full chain
+    class FullChain(str, enum.Enum):
+        Only = '1'
+        Require = '2'
+        Capable = '3'
+
+    # enum for order input by
+    class OrderInputBy(str, enum.Enum):
+        eventsAlignment = '1'
 
     # constructor
     def __init__(self):
@@ -436,7 +453,13 @@ class JediTaskSpec(object):
         if self.splitRule is not None:
             tmpMatch = re.search(self.splitRuleToken['nFilesPerJob']+'=(\d+)',self.splitRule)
             if tmpMatch is not None:
-                return int(tmpMatch.group(1))
+                n = int(tmpMatch.group(1))
+                if self.dynamicNumEvents():
+                    inn = self.get_num_events_per_input()
+                    dyn = self.get_min_granularity()
+                    if inn and dyn and inn > dyn:
+                        n *= (inn // dyn)
+                return n
         return None
 
     # remove nFilesPerJob
@@ -1203,8 +1226,6 @@ class JediTaskSpec(object):
 
     # dynamic number of events
     def dynamicNumEvents(self):
-        if self.getNumEventsPerJob() is not None:
-            return False
         if self.splitRule is not None:
             tmpMatch = re.search(self.splitRuleToken['dynamicNumEvents']+'=(\d+)',self.splitRule)
             if tmpMatch is not None:
@@ -1524,9 +1545,106 @@ class JediTaskSpec(object):
     def on_site_merging(self):
         return self.check_split_rule('onSiteMerging')
 
+    # set full chain flag
+    def set_full_chain(self, mode):
+        var = None
+        if mode == 'only':
+            var = self.FullChain.Only
+        elif mode == 'require':
+            var = self.FullChain.Require
+        elif mode == 'capable':
+            var = self.FullChain.Capable
+        if var:
+            self.setSplitRule('fullChain', var)
+
+    # get full chain flag
+    def get_full_chain(self):
+        if self.splitRule:
+            tmpMatch = re.search(self.splitRuleToken['fullChain']+r'=(\d+)', self.splitRule)
+            if tmpMatch:
+                return tmpMatch.group(1)
+        return None
+
+    # check full chain with mode
+    def check_full_chain_with_mode(self, mode):
+        task_flag = self.get_full_chain()
+        if mode == 'only':
+            if task_flag == self.FullChain.Only:
+                return True
+        elif mode == 'require':
+            if task_flag == self.FullChain.Require:
+                return True
+        elif mode == 'capable':
+            if task_flag == self.FullChain.Capable:
+                return True
+        return False
+
+    # check full chain with nucleus
+    def check_full_chain_with_nucleus(self, nucleus):
+        if self.get_full_chain() and nucleus.get_bare_nucleus_mode():
+            return True
+        return False
+
+    # get RAM for retry
+    def get_ram_for_retry(self, current_ram):
+        if not self.splitRule:
+            return None
+        tmpMatch = re.search(self.splitRuleToken['retryRamOffset'] + r'=(\d+)', self.splitRule)
+        if not tmpMatch:
+            return None
+        offset = int(tmpMatch.group(1))
+        tmpMatch = re.search(self.splitRuleToken['retryRamStep'] + r'=(\d+)', self.splitRule)
+        if tmpMatch:
+            step = int(tmpMatch.group(1))
+        else:
+            step = 0
+        if not current_ram:
+            return current_ram
+        if current_ram < offset:
+            return offset
+        if not step:
+            return current_ram
+        return offset + math.ceil((current_ram - offset) / step) * step
+
+    # get number of events per input
+    def get_num_events_per_input(self):
+        if self.splitRule is not None:
+            tmpMatch = re.search(self.splitRuleToken['nEventsPerInput']+'=(\d+)',self.splitRule)
+            if tmpMatch is not None:
+                return int(tmpMatch.group(1))
+        return None
+
+    def get_max_events_per_job(self):
+        if self.splitRule is not None:
+            tmpMatch = re.search(self.splitRuleToken['maxEventsPerJob']+'=(\d+)',self.splitRule)
+            if tmpMatch is not None:
+                return int(tmpMatch.group(1))
+        return None
+
+    # set order input by
+    def set_order_input_by(self, mode):
+        var = None
+        if mode == 'eventsAlignment':
+            var = self.OrderInputBy.eventsAlignment
+        if var:
+            self.setSplitRule('orderInputBy', var)
+
+    # get full chain flag
+    def order_input_by(self):
+        if self.splitRule:
+            tmpMatch = re.search(self.splitRuleToken['orderInputBy']+r'=(\d+)', self.splitRule)
+            if tmpMatch:
+                if tmpMatch.group(1) == self.OrderInputBy.eventsAlignment:
+                    return 'eventsAlignment'
+        return None
+
+    # check if intermediate task
+    def is_intermediate_task(self):
+        return self.check_split_rule('intermediateTask')
 
 # utils
 
+
 # check if push status changes without class instance
 def push_status_changes(split_rule):
     if not split_rule:
