@@ -35,6 +35,14 @@ class AtlasTaskWithholderWatchDog(WatchDogBase):
         # call refresh
         self.refresh()
 
+    # get process lock
+    def _get_lock(self):
+        return self.taskBufferIF.lockProcess_JEDI(
+                vo=self.vo, prodSourceLabel='managed',
+                cloud=None, workqueue_id=None, resource_name=None,
+                component='AtlasTaskWithholderWatchDog',
+                pid=self.pid, timeLimit=5)
+
     # refresh information stored in the instance
     def refresh(self):
         # work queue mapper
@@ -84,10 +92,12 @@ class AtlasTaskWithholderWatchDog(WatchDogBase):
             tmpSiteSpec = self.siteMapper.getSite(tmpPseudoSiteName)
             tmpSiteName = tmpSiteSpec.get_unified_name()
             # get nQueue and nRunning
-            nRunning = AtlasBrokerUtils.getNumJobs(jobStatPrioMap, tmpSiteName, 'running', gshare)
+            nRunning = AtlasBrokerUtils.getNumJobs(jobStatPrioMap, tmpSiteName, 'running',
+                                                   workQueue_tag=gshare)
             nQueue = 0
             for jobStatus in ['defined', 'assigned', 'activated', 'starting']:
-                nQueue += AtlasBrokerUtils.getNumJobs(jobStatPrioMap, tmpSiteName, jobStatus, gshare)
+                nQueue += AtlasBrokerUtils.getNumJobs(jobStatPrioMap, tmpSiteName, jobStatus,
+                                                      workQueue_tag=gshare)
             # busy sites
             if nQueue > max(cutoff, nRunning*2):
                 busy_sites_list.append(tmpSiteName)
@@ -150,6 +160,8 @@ class AtlasTaskWithholderWatchDog(WatchDogBase):
                         continue
                 # make sql parameters of rses
                 to_exclude_rses = list(busy_rses|blacklisted_rse_set)
+                if not to_exclude_rses:
+                    continue
                 rse_params_list = []
                 rse_params_map = {}
                 for j, rse in enumerate(to_exclude_rses):
@@ -164,6 +176,11 @@ class AtlasTaskWithholderWatchDog(WatchDogBase):
                     "WHERE t.status IN ('ready','running','scouting') AND t.lockedBy IS NULL "
                         "AND t.gshare=:gshare "
                         "AND t.ioIntensity>=:ioIntensity AND t.currentPriority<:currentPriority "
+                        "AND EXISTS ( "
+                            "SELECT * FROM {jedi_schema}.JEDI_Datasets d "
+                            "WHERE d.jediTaskID=t.jediTaskID "
+                                "AND d.type='input' "
+                            ") "
                         "AND NOT EXISTS ( "
                             "SELECT * FROM {jedi_schema}.JEDI_Dataset_Locality dl "
                             "WHERE dl.jediTaskID=t.jediTaskID "
@@ -192,6 +209,11 @@ class AtlasTaskWithholderWatchDog(WatchDogBase):
                         "WHERE t.status IN ('ready','running','scouting') AND t.lockedBy IS NULL "
                             "AND t.gshare=:gshare "
                             "AND t.ioIntensity>=:ioIntensity AND t.currentPriority<:currentPriority "
+                            "AND EXISTS ( "
+                                "SELECT * FROM {jedi_schema}.JEDI_Datasets d "
+                                "WHERE d.jediTaskID=t.jediTaskID "
+                                    "AND d.type='input' "
+                                ") "
                             "AND NOT EXISTS ( "
                                 "SELECT * FROM {jedi_schema}.JEDI_Dataset_Locality dl "
                                 "WHERE dl.jediTaskID=t.jediTaskID "
@@ -217,6 +239,12 @@ class AtlasTaskWithholderWatchDog(WatchDogBase):
             # get logger
             origTmpLog = MsgWrapper(logger)
             origTmpLog.debug('start')
+            # lock
+            got_lock = self._get_lock()
+            if not got_lock:
+                origTmpLog.debug('locked by another process. Skipped')
+                return self.SC_SUCCEEDED
+            origTmpLog.debug('got lock')
             # make tasks pending under certain conditions
             self.do_for_data_locality()
         except Exception:
