@@ -324,11 +324,11 @@ class ContentsFeederThread (WorkerThread):
                                                 nPFN = datasetSpec.getNumRecords()
                                             elif origNumFiles is not None:
                                                 nPFN = origNumFiles
-                                                if 'nEventsPerJob' in taskParamMap and 'nEventsPerFile' in taskParamMap \
+                                                if 'nEventsPerFile' in taskParamMap and taskSpec.get_min_granularity():
+                                                    nPFN = nPFN * taskParamMap['nEventsPerFile'] // taskSpec.get_min_granularity()
+                                                elif 'nEventsPerJob' in taskParamMap and 'nEventsPerFile' in taskParamMap \
                                                         and taskParamMap['nEventsPerFile'] > taskParamMap['nEventsPerJob']:
                                                     nPFN = nPFN * math.ceil(taskParamMap['nEventsPerFile'] / taskParamMap['nEventsPerJob'])
-                                                elif 'nEventsPerFile' in taskParamMap and 'nEventsPerRange' in taskParamMap:
-                                                    nPFN = nPFN * taskParamMap['nEventsPerFile'] // taskParamMap['nEventsPerRange']
                                             elif 'nEvents' in taskParamMap and 'nEventsPerJob' in taskParamMap:
                                                 nPFN = math.ceil(taskParamMap['nEvents'] / taskParamMap['nEventsPerJob'])
                                             elif 'nEvents' in taskParamMap and 'nEventsPerFile' in taskParamMap \
@@ -421,10 +421,10 @@ class ContentsFeederThread (WorkerThread):
                                         elif datasetSpec.isMaster() and datasetSpec.isPseudo() and 'nEvents' in taskParamMap:
                                             # use nEvents as nEventsPerFile for pseudo input
                                             nEventsPerFile = taskParamMap['nEvents']
-                                        if 'nEventsPerJob' in taskParamMap:
+                                        if taskSpec.get_min_granularity():
+                                            nEventsPerRange = taskSpec.get_min_granularity()
+                                        elif 'nEventsPerJob' in taskParamMap:
                                             nEventsPerJob = taskParamMap['nEventsPerJob']
-                                        elif 'nEventsPerRange' in taskParamMap:
-                                            nEventsPerRange = taskParamMap['nEventsPerRange']
                                         if 'tgtNumEventsPerJob' in taskParamMap:
                                             tgtNumEventsPerJob = taskParamMap['tgtNumEventsPerJob']
                                             # reset nEventsPerJob
@@ -472,14 +472,14 @@ class ContentsFeederThread (WorkerThread):
                                             # multiplied by the number of jobs per file for event-level splitting
                                             if nMaxFiles is not None:
                                                 if 'nEventsPerFile' in taskParamMap:
-                                                    if 'nEventsPerJob' in taskParamMap:
+                                                    if taskSpec.get_min_granularity():
+                                                        if taskParamMap['nEventsPerFile'] > taskSpec.get_min_granularity():
+                                                            nMaxFiles *= float(taskParamMap['nEventsPerFile'])/float(taskSpec.get_min_granularity())
+                                                            nMaxFiles = int(math.ceil(nMaxFiles))
+                                                    elif 'nEventsPerJob' in taskParamMap:
                                                         if taskParamMap['nEventsPerFile'] > taskParamMap['nEventsPerJob']:
                                                             nMaxFiles *= float(taskParamMap['nEventsPerFile'])/float(taskParamMap['nEventsPerJob'])
                                                             nMaxFiles = int(math.ceil(nMaxFiles))
-                                                    elif 'nEventsPerRange' in taskParamMap:
-                                                        if taskParamMap['nEventsPerFile'] > taskParamMap['nEventsPerRange']:
-                                                            nMaxFiles *= float(taskParamMap['nEventsPerFile'])/float(taskParamMap['nEventsPerRange'])
-                                                            nMaxFiles = int(math.ceil(nMaxFiles))
                                                 elif 'useRealNumEvents' in taskParamMap:
                                                     # reset nMaxFiles since it is unknown
                                                     nMaxFiles = None
@@ -501,6 +501,12 @@ class ContentsFeederThread (WorkerThread):
                                         skipShortInput = True
                                     else:
                                         skipShortInput = False
+                                    # order by
+                                    if taskSpec.order_input_by() and datasetSpec.isMaster() \
+                                            and not datasetSpec.isPseudo():
+                                        orderBy = taskSpec.order_input_by()
+                                    else:
+                                        orderBy = None
                                     # feed files to the contents table
                                     tmpLog.debug('update contents')
                                     retDB,missingFileList,nFilesUnique,diagMap = \
@@ -533,7 +539,8 @@ class ContentsFeederThread (WorkerThread):
                                                                                      ramCount,
                                                                                      taskSpec,
                                                                                      skipShortInput,
-                                                                                     inputPreStaging)
+                                                                                     inputPreStaging,
+                                                                                     orderBy)
                                     if retDB is False:
                                         taskSpec.setErrDiag('failed to insert files for {0}. {1}'.format(datasetSpec.datasetName,
                                                                                                          diagMap['errMsg']))
@@ -665,15 +672,25 @@ class ContentsFeederThread (WorkerThread):
                     tmp_name = re.sub('/$', '', tmp_name)
                 except Exception:
                     continue
-                if 'prestagingRuleID' in task_params_map:
-                    if tmp_name not in task_params_map['prestagingRuleID']:
-                        continue
-                    rule_id = task_params_map['prestagingRuleID'][tmp_name]
-                else:
-                    rule_id = self.ddmIF.getInterface(task_spec.vo, task_spec.cloud).getActiveStagingRule(
-                        tmp_scope + ':' + tmp_name)
-                    if rule_id is None:
-                        continue
+                try:
+                    if 'prestagingRuleID' in task_params_map:
+                        if tmp_name not in task_params_map['prestagingRuleID']:
+                            continue
+                        rule_id = task_params_map['prestagingRuleID'][tmp_name]
+                    elif 'selfPrestagingRule' in task_params_map:
+                        if not datasetSpec.isMaster() or datasetSpec.isPseudo():
+                            continue
+                        rule_id = self.ddmIF.getInterface(task_spec.vo, task_spec.cloud).make_staging_rule(
+                            tmp_scope + ':' + tmp_name, task_params_map['selfPrestagingRule'])
+                        if not rule_id:
+                            continue
+                    else:
+                        rule_id = self.ddmIF.getInterface(task_spec.vo, task_spec.cloud).getActiveStagingRule(
+                            tmp_scope + ':' + tmp_name)
+                        if rule_id is None:
+                            continue
+                except Exception as e:
+                    return False, 'DDM error : {}'.format(str(e))
                 # request
                 tmp_log.debug('sending request to iDDS for {0}'.format(datasetSpec.datasetName))
                 req = {
