@@ -64,11 +64,6 @@ class JediMaster:
         # setup TaskBuffer I/F
         taskBufferIF = JediTaskBufferInterface()
         taskBufferIF.setupInterface()
-        # setup intra-node message queue broker proxies
-        if hasattr(jedi_config, 'mq') and hasattr(jedi_config.mq, 'configFile') and jedi_config.mq.configFile:
-            from pandajedi.jediorder.JediMsgProcessor import MsgProcAgent
-            mq_agent = MsgProcAgent(config_file=jedi_config.mq.configFile)
-            mb_proxy_dict = mq_agent.start_passive_mode(prefetch_size=999)
         # the list of JEDI knights
         knightList = []
         # setup TaskRefiner
