@@ -237,15 +237,23 @@ class TaskRefinerBase (object):
             taskSpec.container_name = taskParamMap['container_name']
         self.taskSpec = taskSpec
         # set split rule
-        if 'tgtNumEventsPerJob' in taskParamMap:
-            # set nEventsPerJob not respect file boundaries when nFilesPerJob is not used
-            if 'nFilesPerJob' not in taskParamMap:
-                self.setSplitRule(None,taskParamMap['tgtNumEventsPerJob'],JediTaskSpec.splitRuleToken['nEventsPerJob'])
+        if 'nFilesPerJob' not in taskParamMap:
+            if 'tgtNumEventsPerJob' in taskParamMap:
+                # set nEventsPerJob not to respect file boundaries when nFilesPerJob is not used
+                self.setSplitRule(None, taskParamMap['tgtNumEventsPerJob'],
+                                  JediTaskSpec.splitRuleToken['nEventsPerJob'])
+            elif 'nEventsPerInputFile' in taskParamMap and 'nEventsPerJob' in taskParamMap and \
+                    taskParamMap['nEventsPerJob'] >= taskParamMap['nEventsPerInputFile']:
+                # set nFilesPerJob if nEventsPerJob and nEventsPerInputFile are set
+                nFilesPerJob = taskParamMap['nEventsPerJob'] // taskParamMap['nEventsPerInputFile']
+                self.setSplitRule(None, nFilesPerJob, JediTaskSpec.splitRuleToken['nFilesPerJob'])
         self.setSplitRule(taskParamMap,'nFilesPerJob',     JediTaskSpec.splitRuleToken['nFilesPerJob'])
         self.setSplitRule(taskParamMap,'nEventsPerJob',    JediTaskSpec.splitRuleToken['nEventsPerJob'])
         self.setSplitRule(taskParamMap,'nGBPerJob',        JediTaskSpec.splitRuleToken['nGBPerJob'])
         self.setSplitRule(taskParamMap,'nMaxFilesPerJob',  JediTaskSpec.splitRuleToken['nMaxFilesPerJob'])
+        self.setSplitRule(taskParamMap,'maxEventsPerJob',  JediTaskSpec.splitRuleToken['maxEventsPerJob'])
         self.setSplitRule(taskParamMap,'nEventsPerWorker', JediTaskSpec.splitRuleToken['nEventsPerWorker'])
+        self.setSplitRule(taskParamMap,'nEventsPerInputFile', JediTaskSpec.splitRuleToken['nEventsPerInput'])
         self.setSplitRule(taskParamMap,'disableAutoRetry', JediTaskSpec.splitRuleToken['disableAutoRetry'])
         self.setSplitRule(taskParamMap,'nEsConsumers',     JediTaskSpec.splitRuleToken['nEsConsumers'])
         self.setSplitRule(taskParamMap,'waitInput',        JediTaskSpec.splitRuleToken['waitInput'])
@@ -264,6 +272,8 @@ class TaskRefinerBase (object):
         self.setSplitRule(taskParamMap, 'maxNumJobs', JediTaskSpec.splitRuleToken['maxNumJobs'])
         self.setSplitRule(taskParamMap, 'totNumJobs', JediTaskSpec.splitRuleToken['totNumJobs'])
         self.setSplitRule(taskParamMap, 'nChunksToWait', JediTaskSpec.splitRuleToken['nChunksToWait'])
+        self.setSplitRule(taskParamMap, 'retryRamOffset', JediTaskSpec.splitRuleToken['retryRamOffset'])
+        self.setSplitRule(taskParamMap, 'retryRamStep', JediTaskSpec.splitRuleToken['retryRamStep'])
         if 'forceStaged' in taskParamMap:
             taskParamMap['useLocalIO'] = taskParamMap['forceStaged']
         if 'useLocalIO' in taskParamMap:
@@ -388,6 +398,12 @@ class TaskRefinerBase (object):
             self.setSplitRule(None, 1, JediTaskSpec.splitRuleToken['pushJob'])
         if 'onSiteMerging' in  taskParamMap and taskParamMap['onSiteMerging']:
             self.setSplitRule(None, 1, JediTaskSpec.splitRuleToken['onSiteMerging'])
+        if 'fullChain' in taskParamMap:
+            self.taskSpec.set_full_chain(taskParamMap['fullChain'])
+        if 'orderInputBy' in taskParamMap:
+            self.taskSpec.set_order_input_by(taskParamMap['orderInputBy'])
+        if 'intermediateTask' in taskParamMap:
+            self.setSplitRule(None, 1, JediTaskSpec.splitRuleToken['intermediateTask'])
         # work queue
         workQueue = None
         if 'workQueueName' in taskParamMap:
@@ -546,6 +562,25 @@ class TaskRefinerBase (object):
                         for tmpDatasetName in taskParamMap[incexecDS].split(','):
                             if tmpDatasetName not in datasetNameList:
                                 datasetNameList.append(tmpDatasetName)
+                    # consolidation
+                    if len(datasetNameList) > 1 and 'consolidate' in tmpItem:
+                        tmpIF = self.ddmIF.getInterface(self.taskSpec.vo, self.taskSpec.cloud)
+                        if tmpIF:
+                            containerName = tmpItem['consolidate']
+                            tmpStat = tmpIF.registerNewDataset(containerName)
+                            if not tmpStat:
+                                errStr = 'failed to register {}'.format(containerName)
+                                raise JediException.ExternalTempError(errStr)
+                            tmpDsListInCont = tmpIF.listDatasetsInContainer(containerName)
+                            for tmpContName in datasetNameList:
+                                tmpDsNameList = tmpIF.expandContainer(tmpContName)
+                                for tmpDsName in tmpDsNameList:
+                                    if tmpDsName not in tmpDsListInCont:
+                                        tmpStat = tmpIF.addDatasetsToContainer(containerName, [tmpDsName])
+                                        if not tmpStat:
+                                            errStr = 'failed to add {} to {}'.format(tmpDsName, containerName)
+                                            raise JediException.ExternalTempError(errStr)
+                            datasetNameList = [containerName]
                     # loop over all dataset names
                     inDatasetSpecList = []
                     for datasetName in datasetNameList:
@@ -886,7 +921,7 @@ class TaskRefinerBase (object):
     # get parameters for event service merging
     def getParamsForEventServiceMerging(self,taskParamMap):
         # no event service
-        if not self.taskSpec.useEventService() and self.taskSpec.on_site_merging():
+        if not self.taskSpec.useEventService() or self.taskSpec.on_site_merging():
             return None
         # extract parameters
         transPath = 'UnDefined'
@@ -897,12 +932,6 @@ class TaskRefinerBase (object):
             if 'jobParameters' in taskParamMap['esmergeSpec']:
                 jobParameters = jobParameters['esmergeSpec']['jobParameters']
         # return
-        if self.taskSpec.on_site_merging():
-            data = {}
-            data['transPath'] = transPath
-            data['jobParameters'] = jobParameters
-            data['nEventsPerOutputFile'] = jobParameters['nEventsPerOutputFile']
-            return '<___PANDA_MERGE___>' + json.dumps(data)
         return '<PANDA_ESMERGE_TRF>'+transPath+'</PANDA_ESMERGE_TRF>'+'<PANDA_ESMERGE_JOBP>'+jobParameters+'</PANDA_ESMERGE_JOBP>'
 
 
