@@ -1,4 +1,5 @@
 import sys
+import traceback
 
 from pandajedi.jedicore.MsgWrapper import MsgWrapper
 from .TaskSetupperBase import TaskSetupperBase
@@ -64,6 +65,13 @@ class AtlasTaskSetupper (TaskSetupperBase):
                     # DDM backend
                     ddmBackEnd = taskSpec.getDdmBackEnd()
                     tmpLog.info('checking {0}'.format(datasetSpec.datasetName))
+                    # secondary nucleus
+                    nucleusSpec = siteMapper.getNucleus(taskSpec.nucleus)
+                    secNucleusSpecBase = None
+                    if nucleusSpec and not taskSpec.is_intermediate_task():
+                        secondaryNucleus = nucleusSpec.get_secondary_nucleus()
+                        if secondaryNucleus:
+                            secNucleusSpecBase = siteMapper.getNucleus(secondaryNucleus)
                     # check if dataset and container are available in DDM
                     for targetName in [datasetSpec.datasetName,datasetSpec.containerName]:
                         if targetName is None:
@@ -113,6 +121,12 @@ class AtlasTaskSetupper (TaskSetupperBase):
                                         metaData['transient'] = datasetSpec.getTransient()
                                 else:
                                     metaData = None
+                                # use secondary nucleus only for production output
+                                if secNucleusSpecBase and taskSpec.prodSourceLabel in ['managed', 'test'] and \
+                                        targetName == datasetSpec.datasetName and datasetSpec.type == 'output':
+                                    secNucleusSpec = secNucleusSpecBase
+                                else:
+                                    secNucleusSpec = None
                                 # register dataset/container
                                 tmpLog.info('registering {0} with location={1} backend={2} lifetime={3} meta={4}'.format(targetName,
                                                                                                                          location,
@@ -125,7 +139,8 @@ class AtlasTaskSetupper (TaskSetupperBase):
                                     tmpLog.error('failed to register {0}'.format(targetName))
                                     return retFatal
                                 # procedures for user
-                                if userSetup or DataServiceUtils.getDistributedDestination(datasetSpec.storageToken) is not None:
+                                if userSetup or DataServiceUtils.getDistributedDestination(datasetSpec.storageToken) \
+                                        is not None or secNucleusSpec:
                                     # register location
                                     tmpToRegister = False
                                     if userSetup and targetName == datasetSpec.datasetName and datasetSpec.site not in ['',None]:
@@ -139,6 +154,15 @@ class AtlasTaskSetupper (TaskSetupperBase):
                                         userName = None
                                         grouping = 'NONE'
                                         tmpToRegister = True
+                                    elif secNucleusSpec:
+                                        userName = None
+                                        grouping = None
+                                        tmpToRegister = True
+                                        locForRule = siteMapper.getDdmEndpoint(secNucleusSpec.getOnePandaSite(),
+                                                                               datasetSpec.storageToken,
+                                                                               taskSpec.prodSourceLabel,
+                                                                               JobUtils.translate_tasktype_to_jobtype(taskSpec.taskType))
+
                                     if tmpToRegister:
                                         activity = DataServiceUtils.getActivityForOut(taskSpec.prodSourceLabel)
                                         tmpLog.info('registering location={} lifetime={} days activity={} grouping={} '
@@ -169,12 +193,12 @@ class AtlasTaskSetupper (TaskSetupperBase):
                                                     second_copy = True
 
                                                 if second_copy:
-                                                    locForDouble = '(type=SCRATCHDISK)\\notforextracopy=True'
+                                                    locForDouble = '(type=SCRATCHDISK)\\notforextracopy=True\\{}'.format(locForRule)
                                                     tmpMsg  = 'registering double copy '
                                                     tmpMsg += 'location="{0}" lifetime={1}days activity={2} for dataset={3}'.format(locForDouble,lifetime,
                                                                                                                                     activity,targetName)
                                                     tmpLog.info(tmpMsg)
-                                                    tmpStat = ddmIF.registerDatasetLocation(targetName,locForDouble,copies=2,owner=userName,
+                                                    tmpStat = ddmIF.registerDatasetLocation(targetName,locForDouble,copies=1,owner=userName,
                                                                                             lifetime=lifetime,activity=activity,
                                                                                             grouping='NONE',weight='freespace',
                                                                                             ignore_availability=False)
@@ -253,8 +277,9 @@ class AtlasTaskSetupper (TaskSetupperBase):
             # return
             tmpLog.info('done')
             return retOK
-        except Exception:
-            errtype,errvalue = sys.exc_info()[:2]
-            tmpLog.error('doSetup failed with {0}:{1}'.format(errtype.__name__,errvalue))
+        except Exception as e:
+            tmpLog.error('doSetup failed with {}'.format(str(e)))
+            tb = traceback.format_exc()
             taskSpec.setErrDiag(tmpLog.uploadLog(taskSpec.jediTaskID))
+            tmpLog.error(tb)
             return retFatal
